mod connection;
mod inode;
pub mod manager;
mod token;
mod user;

use diesel::{
    connection::SimpleConnection,
    r2d2::{ConnectionManager, Pool, PooledConnection},
    sqlite::SqliteConnection
};
use diesel_migrations::{embed_migrations, EmbeddedMigrations, MigrationHarness};
pub use inode::Inode;
pub use token::Token;
pub use user::{TfaTypes, User, UserRole};

type RawDBConnection = PooledConnection<ConnectionManager<SqliteConnection>>;
pub type DBPool = Pool<ConnectionManager<SqliteConnection>>;
pub use connection::DBConnection;

pub const MIGRATIONS: EmbeddedMigrations = embed_migrations!("./migrations");

#[derive(Debug)]
pub struct ConnectionOptions {}

impl diesel::r2d2::CustomizeConnection<SqliteConnection, diesel::r2d2::Error> for ConnectionOptions {
    fn on_acquire(&self, conn: &mut SqliteConnection) -> Result<(), diesel::r2d2::Error> {
        conn.batch_execute("PRAGMA journal_mode = WAL; PRAGMA synchronous = NORMAL; PRAGMA busy_timeout = 15000;")
            .map_err(diesel::r2d2::Error::QueryError)
    }
}

pub fn build_pool() -> Pool<ConnectionManager<SqliteConnection>> {
    Pool::builder()
        .connection_customizer(Box::new(ConnectionOptions {}))
        .build(ConnectionManager::<SqliteConnection>::new("sqlite.db"))
        .expect("Failed to open db")
}

pub fn run_migrations(db: &mut RawDBConnection) {
    db.run_pending_migrations(MIGRATIONS).expect("Failed to run migrations");
}
